@@ -4,8 +4,10 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
   
   if (isreg) mods <- as.matrix(mods)
   
-  ll.profilet <- function(par,yi,sei,mods) {
+  hessll.profilet <- function(par,yi,sei,mods) {
     isreg <- !missing(mods)
+
+     # browser()
     
     if (any(is.nan(par))) return(NA)
     
@@ -14,13 +16,14 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
     vinv <- par[3]
     if (isreg) xcoef <- matrix(par[4:length(par)],ncol=1)
     
-    if (tau2 < 0.0) return(NA)    
-    if (vinv < 0.0) return(NA)
-    
+    if (tau2 < 0.0) return(NA)
     
     f <- function(nu,oney,onesigma2,onemods,muhat,tau2,vinv,xcoef) {
-         if (isreg) onel <- exp(-(oney-muhat-matrix(onemods,nrow=1) %*% xcoef-nu)^2/(2*onesigma2))*(1.0/sqrt(tau2))*dt(nu/sqrt(tau2),df=1.0/vinv)
+      if ((tau2==0.0)|(vinv<0.0)) onel <- rep(0.0,length(nu))
+      else {
+        if (isreg) onel <- exp(-(oney-muhat-matrix(onemods,nrow=1) %*% xcoef-nu)^2/(2*onesigma2))*(1.0/sqrt(tau2))*dt(nu/sqrt(tau2),df=1.0/vinv)
         else onel <- exp(-(oney-muhat-nu)^2/(2*onesigma2))*(1.0/sqrt(tau2))*dt(nu/sqrt(tau2),df=1.0/vinv)
+      }
       return(onel)        
     }
     calconell <- function(x) {
@@ -28,7 +31,7 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
       onesigma2 <- x[2]
       if (isreg) onemods <- x[3:length(x)]
       else onemods <- NULL
-        theint <- tryCatch({
+       theint <- tryCatch({
           if (isreg) integrate(f,-Inf,Inf,oney=oney,onesigma2=onesigma2,onemods=onemods,muhat=muhat,tau2=tau2,
                              vinv=vinv,xcoef=xcoef,subdivisions = 300L)$value
         else integrate(f,-Inf,Inf,oney=oney,onesigma2=onesigma2,onemods=NULL,muhat=muhat,tau2=tau2,vinv=vinv,
@@ -41,7 +44,7 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
       ll <- log(theint)
       return(ll)
     }
-    if ((vinv==0.0) | (tau2==0.0)){   
+    if (vinv==0.0){   
       w <- 1.0/(tau2+sei^2)
       if (isreg) negll <- 0.5*sum(log(2*pi)+log(1/w)+w*(yi-muhat-as.vector(mods %*% xcoef))^2)
       else negll <- 0.5*sum(log(2*pi)+log(1/w)+w*(yi-muhat)^2)
@@ -54,6 +57,13 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
     if (!is.finite(negll)) negll <- NA
      return(negll)
   }
+  
+  ll.profilet <- function(par,yi,sei,mods) {
+    negll <- hessll.profilet(par,yi,sei,mods)
+    if (is.na(negll)) negll <- 1e100
+    return(negll)
+  }
+  
   # obtain starting values
   if (isreg) {
     start.meta <- rma(yi=yi, sei=sei, mods=as.data.frame(mods), method="DL")
@@ -64,32 +74,43 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
     start.val <- c(start.meta$b[1,1],start.meta$tau2,0.5)
     lower.val <- c(-Inf,0.0,0.0)
   }
+  #browser()
   thenames <- c("muhat","tau2","vinv")
   if (isreg)  thenames <- c(thenames,dimnames(mods)[[2]])
   parnames(ll.profilet) <- thenames
   names(start.val) <- thenames
+  names(lower.val) <- thenames
   
-#   # vinv=0.0 is a special case for some reason, maybe problem in nlminb?
-#   maxfit <- profilenorm.metaplus(yi=yi,sei=sei,mods=mods,justfit=TRUE,plotci=FALSE,slab=NULL)$fittedmodel
-#   if (isreg) maxfit@coef <- c(maxfit@coef[1:2],0.0,maxfit@coef[3:length(maxfit@coef)])
-#   else maxfit@coef <- c(maxfit@coef[1:2],0.0)
-#   names(maxfit@coef)[3] <- "vinv"
-# #  print(logLik(maxfit))
-#   maxll <- logLik(maxfit)
-maxll <- -Inf
-  for (vinv in c(0.0,0.01,0.05,0.1,0.2,0.5,1)) { 
+  # vinv=0.0 is a special case for some reason, maybe problem in optimisation
+  normfit <- profilenorm.metaplus(yi=yi,sei=sei,mods=mods,justfit=TRUE,plotci=FALSE,slab=NULL)$fittedmodel
+  if (isreg) start.null <- c(coef(normfit)[1:2],0.0,coef(normfit)[3:length(coef(normfit))])
+  else start.null <- c(coef(normfit),0.0)
+  names(start.null)[3] <- "vinv"
+  #browser()
+  if (isreg) maxfit <- mymle(ll.profilet,start=start.null,vecpar=TRUE,optimizer="user",
+                                  data=list(yi=yi,sei=sei,mods=mods),
+                                  skip.hessian=TRUE,
+                                  control=list(eval.max=1000),
+                                  lower=lower.val,optimfun=myoptim)
+  else maxfit <- mymle(ll.profilet,start=start.null,vecpar=TRUE,optimizer="user",
+                            data=list(yi=yi,sei=sei),
+                            skip.hessian=TRUE,
+                            control=list(eval.max=1000),
+                            lower=lower.val,optimfun=myoptim)
+  
+  maxll <- logLik(maxfit)
+  for (vinv in c(0.01,0.05,0.1,0.2,0.5,1)) { 
     start.val[3] <- vinv
-    if (isreg) profilet.fit <- suppressWarnings(mle2(ll.profilet,start=start.val,vecpar=TRUE,optimizer="nlminb",
+    if (isreg) profilet.fit <- mymle(ll.profilet,start=start.val,vecpar=TRUE,optimizer="user",
                                                      data=list(yi=yi,sei=sei,mods=mods),
                                                      skip.hessian=TRUE,
                                                      control=list(eval.max=1000),
-                                                     lower=lower.val))
-    else profilet.fit <- suppressWarnings(mle2(ll.profilet,start=start.val,vecpar=TRUE,optimizer="nlminb",
+                                                     lower=lower.val,optimfun=myoptim)
+    else profilet.fit <- mymle(ll.profilet,start=start.val,vecpar=TRUE,optimizer="user",
                                                data=list(yi=yi,sei=sei),
                                                skip.hessian=TRUE,
                                                control=list(eval.max=1000),
-                                               lower=lower.val))
-    #print(logLik(profilet.fit))
+                                               lower=lower.val,optimfun=myoptim)
     if (logLik(profilet.fit) > maxll) {
       maxfit <- profilet.fit
       maxll <- logLik(profilet.fit)
@@ -97,38 +118,31 @@ maxll <- -Inf
   }
   profilet.fit <- maxfit
 # also tau2 and vinv are zero as not identifiable
-  newstart.val <- start.val[c(-2,-3)]
-  newlower.val <- lower.val[c(-2,-3)]
-  if (isreg) profilet.fit2 <- suppressWarnings(mle2(ll.profilet,start=newstart.val,vecpar=TRUE,optimizer="nlminb",
+  newstart.val <- start.val
+  newstart.val[2:3] <- 0.0
+  newlower.val <- lower.val
+  newlower.val[2:3] <- 0.0
+  if (isreg) profilet.fit2 <- mymle(ll.profilet,start=newstart.val,vecpar=TRUE,optimizer="user",
                                                      data=list(yi=yi,sei=sei,mods=mods),
                                                      skip.hessian=TRUE,
                                                      control=list(eval.max=1000),
                                                      lower=newlower.val,
-                                                     fixed=list(tau2=0.0,vinv=0.0)))
-    else profilet.fit2 <- suppressWarnings(mle2(ll.profilet,start=newstart.val,vecpar=TRUE,optimizer="nlminb",
+                                                     fixed=list(tau2=0.0,vinv=0.0),optimfun=myoptim)
+    else profilet.fit2 <- mymle(ll.profilet,start=newstart.val,vecpar=TRUE,optimizer="user",
                                                data=list(yi=yi,sei=sei),
                                                skip.hessian=TRUE,
                                                control=list(eval.max=1000),
                                                  lower=newlower.val,
-                                               fixed=list(tau2=0.0,vinv=0.0)))
-  if(logLik(profilet.fit2) >= logLik(profilet.fit)){
+                                               fixed=list(tau2=0.0,vinv=0.0),optimfun=myoptim)
+  if(logLik(profilet.fit2) > logLik(profilet.fit)){
     profilet.fit@coef <- coef(profilet.fit2)
+    profilet.fit@vcov <- vcov(profilet.fit2)
     profilet.fit@min <- profilet.fit2@min
-    profilet.fit@details$convergence <- profilet.fit2@details$convergence    
-    profilet.fit@details$message <- profilet.fit2@details$message
-  } 
-  if (profilet.fit@details$convergence!=0) {
-# ignore singularity if due to small vinv
-    if (!(((profilet.fit@details$message=="singular convergence (7)") & (profilet.fit@coef[3]<1e-6)) |
-      (profilet.fit@details$message=="false convergence (8)") |
-        (profilet.fit@details$message=="iteration limit reached without convergence (10)"))) {
-      warning(paste("convergence failed: ",profilet.fit@details$message,sep=""))
     }
-  } 
   results <- profilet.fit@coef
   if (!justfit) {
-    if (isreg) thehessian <- hessian(ll.profilet,results,yi=yi,sei=sei,mods=mods)
-    else thehessian <- hessian(ll.profilet,results,yi=yi,sei=sei)
+    if (isreg) thehessian <- hessian(hessll.profilet,results,yi=yi,sei=sei,mods=mods)
+    else thehessian <- hessian(hessll.profilet,results,yi=yi,sei=sei)
     isproblem <- as.numeric(!(is.finite(diag(thehessian)) & (diag(thehessian)!=0.0)))
     isproblem2 <- isproblem*(1:length(results))
     noproblem2 <- (1-isproblem)*(1:length(results))
@@ -140,7 +154,7 @@ maxll <- -Inf
     myse[noproblem2] <- themyse
     if (isreg) whichp <- c(1,4:(3+dim(mods)[2]))
     else whichp <- 1
-    profilet.profile <- suppressWarnings(profilet.profile(profilet.fit,which=whichp,std.err=myse))
+    profilet.profile <- profilet.profile(profilet.fit,which=whichp,std.err=myse)
      profilet.ci <- confint(profilet.profile,method="uniroot")
     if (plotci) plot(profilet.profile)
     
@@ -149,17 +163,19 @@ maxll <- -Inf
     results <- cbind(results,theci)
     pvalues <- rep(NA,length(start.val))
     for (iparm in whichp) {
-      newstart.val <- coef(profilet.fit)[-iparm]
-      newlower.val <- lower.val[-iparm]
+      newstart.val <- coef(profilet.fit)
+      newstart.val[iparm] <- 0.0
+      newlower.val <- lower.val
+      newlower.val[iparm] <- 0.0
       fixedparm <- names(start.val)[iparm]
-      if (isreg) doprofile <- paste("profilet.fit0 <- suppressWarnings(mle2(ll.profilet,start=newstart.val,vecpar=TRUE,\n",
-                                    "optimizer=\"nlminb\",data=list(yi=yi,sei=sei,mods=mods),\n",
+      if (isreg) doprofile <- paste("profilet.fit0 <- mymle(ll.profilet,start=newstart.val,vecpar=TRUE,\n",
+                                    "optimizer=\"user\",data=list(yi=yi,sei=sei,mods=mods),\n",
                                     "skip.hessian=TRUE,\n",
-                                    "lower=newlower.val,fixed=list(",fixedparm,"=0.0)))",sep="")
-      else doprofile <- paste("profilet.fit0 <- suppressWarnings(mle2(ll.profilet,start=newstart.val,vecpar=TRUE,\n",
-                              "optimizer=\"nlminb\",data=list(yi=yi,sei=sei),\n",
+                                    "lower=newlower.val,fixed=list(",fixedparm,"=0.0),optimfun=myoptim)",sep="")
+      else doprofile <- paste("profilet.fit0 <- mymle(ll.profilet,start=newstart.val,vecpar=TRUE,\n",
+                              "optimizer=\"user\",data=list(yi=yi,sei=sei),\n",
                               "skip.hessian=TRUE,\n",
-                              "lower=newlower.val,fixed=list(",fixedparm,"=0.0)))",sep="")
+                              "lower=newlower.val,fixed=list(",fixedparm,"=0.0),optimfun=myoptim)",sep="")
        eval(parse(text=doprofile))
       pvalues[iparm] <- anova(profilet.fit,profilet.fit0)[2,5]
     }
